
=head1 NAME

ll5.pm 

=head1 SYNOPSIS

Statistical library package to calculate the Loglikelihood ratio for 
trigrams. This package should be used with statistic.pl and rank.pl.

=head1 DESCRIPTION

The log-likelihood ratio measures the devitation between the observed data 
and what would be expected if <word1>, <word2>, <word3> and <word4>  were  
independent. The higher the score, the less evidence there is in favor of  
concluding that the words are independent. 

=head1 AUTHORS

Ted Pedersen <tpederse@d.umn.edu>

Bridget Thomson McInnes <bthomson@d.umn.edu>

=head1 BUGS

This measure currently only defined for 4gram data stored in 4x4x4x4
contingency table. 

=head1 SEE ALSO

Mailing List: http://groups.yahoo.com/ngram

=head1 COPYRIGHT

Copyright 2000-2004 by Ted Pedersen and Satanjeev Banerjee and Amruta 
Purandare

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=cut

## this module written by Amruta Purandare, based on ll.pm by Satanjeev 
## Banerjee

package ll5;

#  Make sure that measure5d.pm is available in the PATH. First
#  we check in the directory you are running from, and then we
#  look through the system path. If the module is not found anywhere
#  then abort. 

my $module = "measure5d.pm"; my $modulename = "measure5d.pm";

if( !( -f $modulename ) ) {
    my $found = 0;
    #  Check each of the PATHS to see if the module is there
    foreach (split(/:/, $ENV{PATH})) {
 	$module = $_ . "/" . $modulename;
	if ( -f $module ) { $found = 1; last; }
    }
    # if still not found anywhere, quit!
    if ( ! $found ) { print "Could not find $modulename.\n"; exit; }
}

# Include the module into the current package.    

require $module;
require Exporter;

@ISA = qw ( Exporter );
@EXPORT = qw (initializeStatistic getStatisticName calculateStatistic errorCode errorString);

# function to set up various variables before the actual computation
# starts. also to check if we are being given bigrams, and if our
# frequency combinations are enough to do the computation
sub initializeStatistic
{    
    measure5d::initializeStatistic(@_);
}

# function to calculate the ll value!
sub calculateStatistic
{
    #  The parameters passed into calculateStatistic
    #  need to be passed into getObservedValues for it 
    #  to calculate the Observed values. If there is a 
    #  problem with the observed values, this function 
    #  will return, causing the measure to abort its 
    #  computations and return with a zero code.
    if( !( ($n11111, $n11112, $n11121, $n11122, $n11211, $n11212, $n11221, 
	    $n11222, $n12111, $n12112, $n12121, $n12122, $n12211, $n12212, 
	    $n12221, $n12222, $n21111, $n21112, $n21121, $n21122, $n21211, 
	    $n21212, $n21221, $n21222, $n22111, $n22112, $n22121, $n22122, 
	    $n22211, $n22212, $n22221, $n22222) 
	   = measure5d::getObservedValues(@_) ) ) { return(0); }
    
    #  Check the expected values to ensure that they are not 
    #  equal to zero. Also, the quotient of the observed and 
    #  expected value should not be less than zero. If there 
    #  is a problem with the expected values, this function
    #  will return, causing the measure to abort and return
    #  with a zero code.
    
    if( !( ($m11111, $m11112, $m11121, $m11122, $m11211, $m11212, $m11221, 
	    $m11222, $m12111, $m12112, $m12121, $m12122, $m12211, $m12212, 
	    $m12221, $m12222, $m21111, $m21112, $m21121, $m21122, $m21211, 
	    $m21212, $m21221, $m21222, $m22111, $m22112, $m22121, $m22122, 
	    $m22211, $m22212, $m22221, $m22222)
	   = measure5d::getExpectedValues() ) ) { return(0); }
    
    #  Get the total number of bigrams; no need to check 
    #  if this is correct because this value was already 
    #  checked and computed in measuredd::initalizeStatistic.
    #  getTotalTrigrams simply reads an existing value. 
    
    my $nppppp = measure5d::getTotal5grams();
    
    #  Now for the calculations!!
   
    $logLikelihood = 0;
   
    # dont want ($n111 / $e111) to be 0 or less! flag error if so!
    if ( $n11111 ) { { $logLikelihood += $n11111 * log ( $n11111 / $m11111 ); }}
    if ( $n11112 ) { { $logLikelihood += $n11112 * log ( $n11112 / $m11112 ); }}
    if ( $n11121 ) { { $logLikelihood += $n11121 * log ( $n11121 / $m11121 ); }}
    if ( $n11122 ) { { $logLikelihood += $n11122 * log ( $n11122 / $m11122 ); }}
    if ( $n11211 ) { { $logLikelihood += $n11211 * log ( $n11211 / $m11211 ); }}
    if ( $n11212 ) { { $logLikelihood += $n11212 * log ( $n11212 / $m11212 ); }}
    if ( $n11221 ) { { $logLikelihood += $n11221 * log ( $n11221 / $m11221 ); }}
    if ( $n11222 ) { { $logLikelihood += $n11222 * log ( $n11222 / $m11222 ); }}
    if ( $n12111 ) { { $logLikelihood += $n12111 * log ( $n12111 / $m12111 ); }}
    if ( $n12112 ) { { $logLikelihood += $n12112 * log ( $n12112 / $m12112 ); }}
    if ( $n12121 ) { { $logLikelihood += $n12121 * log ( $n12121 / $m12121 ); }}
    if ( $n12122 ) { { $logLikelihood += $n12122 * log ( $n12122 / $m12122 ); }}
    if ( $n12211 ) { { $logLikelihood += $n12211 * log ( $n12211 / $m12211 ); }}
    if ( $n12212 ) { { $logLikelihood += $n12212 * log ( $n12212 / $m12212 ); }}
    if ( $n12221 ) { { $logLikelihood += $n12221 * log ( $n12221 / $m12221 ); }}
    if ( $n12222 ) { { $logLikelihood += $n12222 * log ( $n12222 / $m12222 ); }}
    if ( $n21111 ) { { $logLikelihood += $n21111 * log ( $n21111 / $m21111 ); }}
    if ( $n21112 ) { { $logLikelihood += $n21112 * log ( $n21112 / $m21112 ); }}
    if ( $n21121 ) { { $logLikelihood += $n21121 * log ( $n21121 / $m21121 ); }}
    if ( $n21122 ) { { $logLikelihood += $n21122 * log ( $n21122 / $m21122 ); }}
    if ( $n21211 ) { { $logLikelihood += $n21211 * log ( $n21211 / $m21211 ); }}
    if ( $n21212 ) { { $logLikelihood += $n21212 * log ( $n21212 / $m21212 ); }}
    if ( $n21221 ) { { $logLikelihood += $n21221 * log ( $n21221 / $m21221 ); }}
    if ( $n21222 ) { { $logLikelihood += $n21222 * log ( $n21222 / $m21222 ); }}
    if ( $n22111 ) { { $logLikelihood += $n22111 * log ( $n22111 / $m22111 ); }}
    if ( $n22112 ) { { $logLikelihood += $n22112 * log ( $n22112 / $m22112 ); }}
    if ( $n22121 ) { { $logLikelihood += $n22121 * log ( $n22121 / $m22121 ); }}
    if ( $n22122 ) { { $logLikelihood += $n22122 * log ( $n22122 / $m22122 ); }}
    if ( $n22211 ) { { $logLikelihood += $n22211 * log ( $n22211 / $m22211 ); }}
    if ( $n22212 ) { { $logLikelihood += $n22212 * log ( $n22212 / $m22212 ); }}
    if ( $n22221 ) { { $logLikelihood += $n22221 * log ( $n22221 / $m22221 ); }}
    if ( $n22222 ) { { $logLikelihood += $n22222 * log ( $n22222 / $m22222 ); }}
    
    print "$n11222 : $m11222\n";
    return (2 * $logLikelihood );
}


# function to return the error code of the last operation and reset
# error code. useful if the error can be recovered from!
sub errorCode 
{ 
    return measure5d::errorCode();
}

# function to return the error message of the last operation and reset
# the message string. useful if error can be recovered from!
sub errorString
{
    return measure5d::errorString();
}

# function to return the name of this statistic
sub getStatisticName
{
    return "Loglikelihood4";
}

1;

